{-| This should probably be in a different directory.
-}

module TypeChecker.Haskell where

import Data.Map as Map hiding (Map)

import Syntax.Internal
import Syntax.Values
import Syntax.Scope

import TypeChecker.Values
import TypeChecker.UnfoldConstants
import TypeChecker.Monad
import TypeChecker.Monad.Errors
import TypeChecker.Errors
import TypeChecker.Abstractions

import Compilers.Run

import Utils.Error
import Utils.Monads

data HEnv = HEnv { hDefinitions :: Map Name Def
		 , hScope	:: ScopeEnv
		 }

getHName :: HEnv -> String -> Name
getHName henv s = getName (hScope henv) s

isName :: HEnv -> String -> Name -> Bool
isName henv s x = getHName henv s == x

apply henv = nfApply (hDefinitions henv)

-- Class definitions ------------------------------------------------------

class Extractable a where
    toHaskell :: HEnv -> NF -> a

class Injectable a where
    fromHaskell	:: HEnv -> a -> NF

-- Value ------------------------------------------------------------------

instance Extractable NF where
    toHaskell _ = id

instance Injectable NF where
    fromHaskell _ = id

-- a -> b -----------------------------------------------------------------

instance (Injectable a, Extractable b) => Extractable (a -> b) where
    toHaskell henv v x = toHaskell henv $ apply henv v [fromHaskell henv x]

instance (Extractable a, Injectable b) => Injectable (a -> b) where
    fromHaskell henv f =
	NLam (
	    Prim "" 1 (\ [v] -> fromHaskell henv $ f $ toHaskell henv v) []
	) []

-- Int --------------------------------------------------------------------

instance Extractable Int where
    toHaskell henv v =
	case v of
	    NCon zero [] | isName henv "zero" zero -> 0
	    NCon suc [w] | isName henv "suc"  suc  -> 1 + toHaskell henv w
	    _					   ->
		error $ "Not a canonical number: " ++ show v

instance Injectable Int where
    fromHaskell henv 0 = NCon (getHName henv "zero") []
    fromHaskell henv n = NCon (getHName henv "suc") [fromHaskell henv (n - 1)]

-- Bool -------------------------------------------------------------------

instance Extractable Bool where
    toHaskell henv v =
	case v of
	    NCon true []  | isName henv "true"  true  -> True
	    NCon false [] | isName henv "false" false -> False
	    _					      ->
		error $ "Not a canonical boolean: " ++ show v

-- (a,b) ------------------------------------------------------------------

instance (Extractable a, Extractable b) => Extractable (a,b) where
    toHaskell henv v =
	case v of
	    NPair v w -> (toHaskell henv v, toHaskell henv w)
	    _	      -> error $ "Not a canonical pair: " ++ show v

-- [a] --------------------------------------------------------------------

instance Extractable a => Extractable [a] where
    toHaskell henv v =
	case v of
	    NCon nil []
		| isName henv "nil" nil -> []
	    NCon cons [x,xs]
		| isName henv "::" cons -> toHaskell henv x : toHaskell henv xs
	    _				-> error $ "Not a canonical list: " ++ show v

