
module TypeChecker.Monad.Context where

import Control.Monad
import Control.Monad.Reader

import Syntax.Internal
import Syntax.Values

import TypeChecker.Monad
import TypeChecker.Monad.Errors
import TypeChecker.Context
import TypeChecker.Errors
import TypeChecker.Abstractions
import TypeChecker.Monad.Rewriting

import Utils.Monads
import Utils.Map hiding (map)
import Utils.Error
import Utils.Lists

-- Context stuff ----------------------------------------------------------

getContext :: TC Context
getContext = liftM envContext ask

getBoundContext :: TC [(Name, Type)]
getBoundContext =
    do	gamma <- getContext
	return [ (x, t) | (x, Abstract t) <- gamma ]

-- | Get the value of a name in the context.
lookupVar :: Name -> TC (Value, Type)
lookupVar x =
    do	gamma <- getContext
	case findInContext x gamma of
	    Just (n, Abstract t)    -> return (vBId n, raiseBy (n + 1) t)
	    Just (n, Defined v t)   -> return $ raiseBy n (v,t)
	    _			    -> failure $ variableNotBoundError x

-- | Get the value of a bound variable (if it's bound). Postcondition: the
--   value is a vBId.
lookupBoundVar :: Name -> TC (Maybe Value)
lookupBoundVar x =
    do	gamma <- getContext
	case findInContext x gamma of
	    Just (n, Abstract t)    -> return $ Just $ vBId n
	    _			    -> return Nothing

-- | The type has to be valid in the empty context.
freshParameter :: Type -> (Value -> TC a) -> TC a
freshParameter t k =
    do	gamma <- getContext
	local (updateContext $ const $ gamma ++ [(DontCare,Abstract t)])
	      (k $ vBId (nofBoundVars gamma))

-- Note: don't export
updateContext :: (Context -> Context) -> Environment -> Environment
updateContext f env = env { envContext = f $ envContext env }

inContext :: Context -> TC a -> TC a
inContext gamma = local (updateContext $ const gamma) . clearLocalRewriteRules

extendContext :: Name -> ContextEntry -> TC a -> TC a
extendContext x v = extendContextALot [(x,v)]

extendContextALot :: Context -> TC a -> TC a
extendContextALot ctx = local (updateContext (ctx++)) . raiseLocalRewriteRules k
    where
	k = length [ () | (_,Abstract _) <- ctx ]

extendContextWithTel :: [(Name, Type)] -> TC a -> TC a
extendContextWithTel []		 = id
extendContextWithTel ((x,t):tel) =
    extendContext x (Abstract t) . extendContextWithTel tel

isBound :: Name -> TC Bool
isBound DontCare = return False
isBound x	 = do	gamma <- getContext
			return $ x `elem` map fst gamma

